class ADN {
  char [] brin1, brin2 ;

  /************************************
   * constructeur de chaine aléatoires complémentaires 
   *
   * entrée :
   *   - n : taille de la chaine d'ADN
   *************************************/  
  ADN(int n) {
    brin1 = new char[n];
    brin2 = new char[n];
    
    for(int i = 0; i < n; i++) {
      setAllele(i, alleleRnd());
    }
  }
  
  /************************************
   * constructeur de chaine aléatoires complémentaires 
   *
   * entrée :
   *   - n : taille de la chaine d'ADN
   *   - p : taux d'erreur dans la base complémentaire
   *************************************/  
  ADN(int n, float p) {
    brin1 = new char[n];
    brin2 = new char[n];
    
    for(int i = 0; i < n; i++) {
      setAllele(i, alleleRnd(), p);
    }
  }
  
  /************************************
   * affecte l'allele à la position désignée de manière correcte 
   *
   * entrée :
   *   - i : position de l'allele
   *   - c : valeur de l'allele
   *
   * sortie :
   *   - aucune
   *************************************/  
  void setAllele(int i, char c) {
    switch (c) {
      case 'A' : 
        brin1[i] = 'A';
        brin2[i] = 'T';
        break;

      case 'T' : 
        brin1[i] = 'T';
        brin2[i] = 'A';
        break;

      case 'G' : 
        brin1[i] = 'G';
        brin2[i] = 'C';
        break;

      case 'C' : 
        brin1[i] = 'C';
        brin2[i] = 'G';
        break;
    }
  }
    
  /************************************
   * calcule l'indice du minimum du tableau 
   *
   * entrée :
   *   - i : position de l'allele
   *   - c : valeur de l'allele
   *   - p : taux d'erreur dans la base complémentaire
   *
   * sortie :
   *   - aucune
   *************************************/  
  void setAllele(int i, char c, float p) {
    switch (c) {
      case 'A' : 
        brin1[i] = 'A';
        if (random(1.0) > p)
          brin2[i] = 'T';
        else
          brin2[i] = alleleRnd();
        break;

      case 'T' : 
        brin1[i] = 'T';
        if (random(1.0) > p)
          brin2[i] = 'A';
        else
          brin2[i] = alleleRnd();
        break;

      case 'G' : 
        brin1[i] = 'G';
        if (random(1.0) > p)
          brin2[i] = 'C';
        else
          brin2[i] = alleleRnd();
        break;

      case 'C' : 
        brin1[i] = 'C';
        if (random(1.0) > p)
          brin2[i] = 'G';
        else
          brin2[i] = alleleRnd();
        break;
    }
  }
    
  /************************************
   * donne un allele maière équiprobable 
   *
   * entrée :
   *   - aucune
   *
   * sortie :
   *   - caractère représentant l'allele
   *************************************/  
  char alleleRnd() {
    switch (int(random(4))) {
      case 0 : return 'A';
      case 1 : return 'T';
      case 2 : return 'G';
      case 3 : return 'C';
      default : return 'X';
    } 
  }

  /************************************
   * dénombrement de ATA 
   *
   * entrée :
   *   - aucune
   *
   * sortie :
   *   - nombre d'occurence
   *************************************/  
  int nbIsoleucine() {
    int i  = 0;
    int nb = 0;
    
    while (i < brin1.length - 2) {
      if (brin1[i] == 'A' && brin1[i+1] == 'T' && brin1[i+2] == 'A')
        nb++;
      i += 3;
    }
    
    return nb;
  }

  /************************************
   * dénombrement de ATA 
   *
   * entrée :
   *   - aucune
   *
   * sortie :
   *   - nombre d'occurence
   *************************************/  
  int nbErreur() {
    int nb = 0;
    
    for(int i = 0; i < brin1.length; i++) {
      if ((brin1[i] == 'A' && brin2[i] != 'T') ||
          (brin1[i] == 'T' && brin2[i] != 'A') ||
          (brin1[i] == 'G' && brin2[i] != 'C') ||
          (brin1[i] == 'C' && brin2[i] != 'G') 
         )
        nb++;
    }
    
    return nb;
  }

  /************************************
   * chaine pour l'affichage des 2 brins 
   *
   * entrée :
   *   - aucune
   *
   * sortie :
   *   - chaine de caractères 
   *************************************/  
  public String toString() {
    String s = "";
    
    for(int i = 0; i < brin1.length; i++)
       s += brin1[i];
       
    s += "\n";
    
    for(int i = 0; i < brin2.length; i++)
       s += brin2[i];
       
    return s;
  }
}

void setup() {
  ADN chaine1 = new ADN(30);
  ADN chaine2 = new ADN(30, 0.5);
  ADN chaine3 = new ADN(1000);
  
  println(chaine1);
  println();
  println(chaine2);
  println();
  println(chaine3);
  
  println("nombre d'isoleucine chaine 3 : " + chaine3.nbIsoleucine());
  println("nombre d'erreur chaine 1 : " + chaine1.nbErreur());
  println("nombre d'erreur chaine 2 : " + chaine2.nbErreur());

}


